%{
#include <aterm2.h>

extern int yylineno;
extern char file_name[256];
extern FILE *yyin;

ATerm parse_tree;

void yyerror(char *msg)
{
  fprintf(stderr, "%s: line %d - %s\n", file_name, yylineno + 1, msg);
  exit(1);
}

%}

%union{
   int       num;
   double    real;
   char      *string;
   ATerm     term;
   ATermList list;
}

%{

/* int yylex(void); */

int yylex(YYSTYPE *lvalp);
int yyparse(void);

int parse()
{
   int result;
   result = yyparse();
   return result;
}

#define get_string(x) ATgetName(ATgetAFun(x))

#define mk_appl(x, y, z) (ATerm)ATmakeApplList (ATmakeAFun(get_string(x), ATgetLength((y)), z), (y))

%}

%token <term>   ID
%token <term>   QNAME
%token <term>   OPERATOR
%token <num>    INTCONST
%token <real>   DBLCONST
%token          MTLIST 
%token          LCONSTR
%token  <term>  MINUS  

%type <term>    pterm
%type <list>    lpterm


%start start
 
%pure_parser

%%

start : ID '.' pterm '.'	{ parse_tree = $3; }
      ;

pterm : 
        ID			{ $$ = $1; }
      | ID '(' lpterm ')'       { $$ = mk_appl($1, $3, ATfalse); 
				  ATfprintf(stderr, "pterm: %t\n", $$);
                                }
      | QNAME    '(' lpterm ')' { $$ = mk_appl($1, $3, ATtrue); }
      | OPERATOR '(' lpterm ')' { $$ = mk_appl($1, $3, ATtrue); }
      | MINUS    '(' lpterm ')' { $$ = mk_appl($1, $3, ATtrue); }
      | LCONSTR  '(' lpterm ')' { $$ = (ATerm)$3; }
      | LCONSTR                 { $$ = (ATerm) ATmakeString("'.'"); }
      | MTLIST			{ $$ = (ATerm)ATempty; }
      | '(' pterm ')' 	        { $$ = $2; }
      | QNAME 			{ $$ = $1; }
      | MINUS    		{ $$ = (ATerm) ATmakeString("-"); }
      | MINUS INTCONST		{ $$ = (ATerm)ATmakeInt(- $2); }
      | INTCONST		{ $$ = (ATerm)ATmakeInt($1); }
      | MINUS DBLCONST		{ $$ = (ATerm)ATmakeReal(- $2); }
      | DBLCONST		{ $$ = (ATerm)ATmakeReal($1); }
      ;
//      | MINUS			{ $$ = ; }
//      ;

lpterm : pterm			{ $$ = ATmakeList1($1); ATfprintf(stderr, "lpterm: %t\n", $$);}
       | pterm ',' lpterm	{ $$ = ATinsert($3, $1); ATfprintf(stderr, "lpterm: %t\n", $$);}
       ;
